using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using System.Linq;

public class Morphing : MonoBehaviour
{
    bool started = false;
    ParticleSystem source;
    public ParticleSystem target;
    public float delay = 0.1f;
    public float duration = 3;
    public bool changeColor = true;
    public bool changeSize = true;
    public bool withSourceChildren = false;
    public bool withTargetChildren = false;
    public bool calculateDistances = true;
    List<ParticleSystem.Particle> sourceParticles = new List<ParticleSystem.Particle>();
    List<ParticleSystem.Particle> targetParticles = new List<ParticleSystem.Particle>();
    List<ParticleSystem.Particle> startParticles = new List<ParticleSystem.Particle>();
    List<ParticleSystem.Particle> finalParticles = new List<ParticleSystem.Particle>();
    float timer;
    bool done = false;

    void OnEnable()
    {
        if(done == false)
        {
            Invoke("StartMorphing", delay);
            done = true;
        }
    }

    void StartMorphing()
    {
        timer = 0;
        source = GetComponent<ParticleSystem>();
        if (withSourceChildren == true)
        {
            foreach (ParticleSystem ps in source.GetComponentsInChildren<ParticleSystem>(false))
            {
                ParticleSystem.Particle[] sourceParticlesArray = new ParticleSystem.Particle[ps.particleCount];
                ps.GetParticles(sourceParticlesArray, ps.particleCount);
                ParticleSystem.Particle[] empty = new ParticleSystem.Particle[0];
                ps.SetParticles(empty);
                sourceParticles.AddRange(sourceParticlesArray);
            }
            source.SetParticles(sourceParticles.ToArray());
        }
        else
        {
            ParticleSystem.Particle[] sourceParticlesArray = new ParticleSystem.Particle[source.particleCount];
            source.GetParticles(sourceParticlesArray, sourceParticlesArray.Length);
            sourceParticles.AddRange(sourceParticlesArray);
        }
        if (withTargetChildren == true)
        {
            foreach (ParticleSystem ps in target.GetComponentsInChildren<ParticleSystem>(false))
            {
                ParticleSystem.Particle[] targetParticlesArray = new ParticleSystem.Particle[ps.particleCount];
                ps.GetParticles(targetParticlesArray, ps.particleCount);
                ParticleSystem.Particle[] empty = new ParticleSystem.Particle[0];
                ps.SetParticles(empty);
                targetParticles.AddRange(targetParticlesArray);
            }
            target.SetParticles(targetParticles.ToArray());
        }
        else
        {
            ParticleSystem.Particle[] targetParticlesArray = new ParticleSystem.Particle[target.particleCount];
            target.GetParticles(targetParticlesArray, targetParticlesArray.Length);
            targetParticles.AddRange(targetParticlesArray);
        }
        if(calculateDistances == true)
        {
            List<ParticleSystem.Particle> originalTargetParticles = new List<ParticleSystem.Particle>();
            originalTargetParticles.AddRange(targetParticles);
            foreach (ParticleSystem.Particle p in sourceParticles)
            {
                if (targetParticles.Count != 0)
                {
                    List<float> distances = new List<float>();
                    foreach (ParticleSystem.Particle tp in targetParticles)
                    {
                        distances.Add(Vector3.Distance(transform.TransformPoint(p.position), target.transform.TransformPoint(tp.position)));
                    }
                    float smallestDistance = distances.Min();
                    startParticles.Add(p);
                    finalParticles.Add(targetParticles[distances.IndexOf(smallestDistance)]);
                    targetParticles.RemoveAt(distances.IndexOf(smallestDistance));
                }
                else
                {
                    targetParticles.AddRange(originalTargetParticles);
                    List<float> distances = new List<float>();
                    foreach (ParticleSystem.Particle tp in targetParticles)
                    {
                        distances.Add(Vector3.Distance(transform.TransformPoint(p.position), target.transform.TransformPoint(tp.position)));
                    }
                    float smallestDistance = distances.Min();
                    startParticles.Add(p);
                    finalParticles.Add(targetParticles[distances.IndexOf(smallestDistance)]);
                    targetParticles.RemoveAt(distances.IndexOf(smallestDistance));
                }
            }
        }
        else
        {
            if(sourceParticles.Count == 0)
            {
                Debug.Log("Source particles count is 0. Did you forgot to check \"With Source Children\" checkbox");
                return;
            }
            if (targetParticles.Count == 0)
            {
                Debug.Log("Target particles count is 0. Did you forgot to check \"With Target Children\" checkbox");
                return;
            }
            int repeatTimes = (int) Mathf.Ceil((float) sourceParticles.Count / targetParticles.Count);
            for(int i = 0; i < repeatTimes; i ++)
            {
                finalParticles.AddRange(targetParticles);
            }
            startParticles = sourceParticles;
        }
        started = true;
    }

    void Update()
    {
        if(started == true)
        {
            timer += Time.deltaTime;
            if (timer <= duration)
            {
                ParticleSystem.Particle[] sourceParticlesArray = sourceParticles.ToArray();
                for (int i = 0; i < sourceParticlesArray.Length; i++)
                {
                    sourceParticlesArray[i].position = transform.InverseTransformPoint(Vector3.Lerp(transform.TransformPoint(startParticles[i].position), target.transform.TransformPoint(finalParticles[i].position), timer / duration));
                    if(changeColor)
                        sourceParticlesArray[i].startColor = Color.Lerp(startParticles[i].startColor, finalParticles[i].startColor, timer / duration);
                    if(changeSize)
                        sourceParticlesArray[i].startSize = startParticles[i].startSize + (finalParticles[i].startSize - startParticles[i].startSize) * (timer / duration);
                }
                source.SetParticles(sourceParticlesArray);
            }
        }
    }
}
