using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using System.IO;

public class SaveEffect : MonoBehaviour
{
    List<ParticleSystem> effect = new List<ParticleSystem>();
    int frame;
    string path;
    public int DurationInTicks = 100;
    public float delay = 0;
    List<string> linesToSave = new List<string>();
    void OnEnable()
    {
        Invoke("StartRecording", delay);
    }
    void StartRecording()
    {
        Time.captureFramerate = 60;
        frame = 0;
        SaveParticles();
        InvokeRepeating("SaveParticles", 0.05f, 0.05f);
    }
    void SaveParticles()
    {
        if (frame > DurationInTicks) return;
        effect.Clear();
        effect.AddRange(GetComponentsInChildren<ParticleSystem>(true));
        Debug.Log(frame / 20);
        linesToSave.Add(frame + ":");
        foreach (ParticleSystem p in effect)
        {
            if (p.gameObject.activeSelf && p.particleCount > 0)
            {
                ParticleSystem.Particle[] particles = new ParticleSystem.Particle[p.particleCount];
                p.GetParticles(particles, p.particleCount);
                for (int i = 0; i < p.particleCount; i++)
                {
                    try
                    {
                        particles[i].GetCurrentColor(p);
                    }
                    catch
                    {
                    }
                    if (particles[i].GetCurrentColor(p).a > 0)
                    {
                        if(p.main.simulationSpace == ParticleSystemSimulationSpace.Local)
                        {
                            if (p.shape.meshRenderer != null)
                            {
                                linesToSave.Add(p.shape.meshRenderer.transform.TransformPoint(particles[i].position).x + ";" + p.shape.meshRenderer.transform.TransformPoint(particles[i].position).y + ";" + p.shape.meshRenderer.transform.TransformPoint(particles[i].position).z * -1 + ";" + particles[i].GetCurrentColor(p).r + ";" + particles[i].GetCurrentColor(p).g + ";" + particles[i].GetCurrentColor(p).b + ";" + particles[i].GetCurrentSize(p));
                            }
                            else if (p.shape.skinnedMeshRenderer != null)
                            {
                                linesToSave.Add(p.shape.skinnedMeshRenderer.rootBone.TransformPoint(particles[i].position).x + ";" + p.shape.skinnedMeshRenderer.rootBone.TransformPoint(particles[i].position).y + ";" + p.shape.skinnedMeshRenderer.rootBone.TransformPoint(particles[i].position).z * -1 + ";" + particles[i].GetCurrentColor(p).r + ";" + particles[i].GetCurrentColor(p).g + ";" + particles[i].GetCurrentColor(p).b + ";" + particles[i].GetCurrentSize(p));
                            }
                            else
                            {
                                linesToSave.Add(p.transform.TransformPoint(particles[i].position).x + ";" + p.transform.TransformPoint(particles[i].position).y + ";" + p.transform.TransformPoint(particles[i].position).z * -1 + ";" + particles[i].GetCurrentColor(p).r + ";" + particles[i].GetCurrentColor(p).g + ";" + particles[i].GetCurrentColor(p).b + ";" + particles[i].GetCurrentSize(p));
                            }
                        }
                        else if(p.main.simulationSpace == ParticleSystemSimulationSpace.World)
                        {
                            linesToSave.Add(particles[i].position.x + ";" + particles[i].position.y + ";" + particles[i].position.z * -1 + ";" + particles[i].GetCurrentColor(p).r + ";" + particles[i].GetCurrentColor(p).g + ";" + particles[i].GetCurrentColor(p).b + ";" + particles[i].GetCurrentSize(p));
                        }
                        else if(p.main.simulationSpace == ParticleSystemSimulationSpace.Custom)
                        {
                            linesToSave.Add(p.main.customSimulationSpace.TransformPoint(particles[i].position).x + ";" + p.main.customSimulationSpace.TransformPoint(particles[i].position).y + ";" + p.main.customSimulationSpace.TransformPoint(particles[i].position).z * -1 + ";" + particles[i].GetCurrentColor(p).r + ";" + particles[i].GetCurrentColor(p).g + ";" + particles[i].GetCurrentColor(p).b + ";" + particles[i].GetCurrentSize(p));
                        }
                    }
                }
            }
        }
        if(frame == DurationInTicks)
        {
            SaveFile();
        }
        frame += 1;
    }
    void SaveFile()
    {
        Directory.CreateDirectory("Assets/Effects");
        path = "Assets/Effects/" + gameObject.name + ".upe";
        File.WriteAllText(path, "");
        Debug.Log("Saving effect");
        StreamWriter writer = new StreamWriter(path, true);
        foreach(string s in linesToSave)
        {
            writer.WriteLine(s);
        }
        writer.Close();
        Debug.Log("Effect saved! You can exit play mode!");
    }
}
